const singleNumRegexp = /\w_(\d+)/;
const doubleNumReg = /.*_(\d+)_(\d+)$/;

// TODO: избавиться от копипасты с rating.js!!!

class Base {

    static parseid($cell) {
        singleNumRegexp.lastIndex = 0;
        let found = singleNumRegexp.exec($cell.attr('id'));
        return found ? parseInt(found[1]) : 0;
    }

    static parseClass($cell) {
        singleNumRegexp.lastIndex = 0;
        let found = singleNumRegexp.exec($cell.attr('class'));
        return found ? parseInt(found[1]) : 0;
    }

    static parseIdText(id) {
        let found = singleNumRegexp.exec(id);
        return found ? parseInt(found[1]) : 0;
    }

    static parsePosition($cell) {
        doubleNumReg.lastIndex = 0;
        let [col, row] = doubleNumReg.exec($cell.attr('id')).splice(1, 2);
        return [ parseInt(col), parseInt(row) ];
    }

    static parseSettings() {
        let $discipline = $('#json_discipline');

        let settings = {};

        settings.discipline = {};
        let disciplineRaw = $.parseJSON($discipline.html() || '{}');
        Object.keys(disciplineRaw).forEach((keyRaw) => {
            let key = keyRaw.slice(0, 1).toLowerCase() + keyRaw.slice(1);
            if (keyRaw == "ID") key = 'id';

            let value = disciplineRaw[keyRaw];

            settings.discipline[key] = value;
        });

        $discipline.remove();

        return settings;
    }

    static parseRate($cell) {
        let def = 0;
        if ($cell.hasClass('static')) {
            return parseInt($cell.text() || def);
        } else {
            return $cell.children('input').prop('checked')?1:0;
        }
    }

}

class Cursor {
    constructor() {
        this.col = 0;
        this.row = 0;
    }

    update(col, row) {
        this.col = col;
        this.row = row;
    }

    get() {
        return [this.col, this.row];
    }

    parseLocation($cell) {
        let [col, row] = Base.parsePosition($cell);
        this.update(col, row);
    }

    isEmpty() {
        return (this.col == 0) && (this.row == 0);
    }
}

class Cell {
    constructor() {
        this.update(0, 0);
    }

    update(recordBook, lesson) {
        this.recordBook = recordBook;
        this.lesson = lesson;
    }
}

class CellInfo {
    constructor(cursor, cell) {
        this.cursor = cursor;
        this.cell = cell;

        this.journalHead = $('#journalHead');
    }

    select($cell) {
        this.cursor.parseLocation($cell);

        let [col] = this.cursor.get();
        let $curLessonHead = this.journalHead.children(`.lesson_col_${col}`);
        let recordBook = Base.parseid($cell.siblings('.name'));
        let lesson = Base.parseid($curLessonHead);
        this.cell.update(recordBook, lesson);
    }

    reset() {
        this.cell.update(0, 0);
    }
}

class Journal {
    constructor() {
        this.settings = Base.parseSettings();
        this.filter = new GroupFilter(this.settings, $('#studentsAttendance'), $('#groupSelector'));
        this.lessonFilter = new LessonFilter(this.settings, $('#studentsAttendance'), $('#lessonSelector'), this);
        this.cell = new Cell();
        this.cursor = new Cursor();
        this.cellInfo = new CellInfo(this.cursor, this.cell);
        //this.lessonCreator = new LessonCreator($('.lessoncreate'));

        let self = this;
        $('#studentsAttendance tbody').on('change', '.checkbox', function () {
            self.cellInfo.select($(this));
            self.setAttendance($(this));
            self.cellInfo.reset();
        });


        let $delete_lesson = $(".delete_lesson");
        let $switcher = $(".delete_button_switcher");
        $delete_lesson.hide();
        $(".rateHeader").on("click", ".delete_button_switcher", function () {
            if ($(".delete_button_switcher").hasClass("delete_mode")) {
                $(".delete_lesson").hide();
                $(".delete_button_switcher").removeClass("delete_mode")
            } else {
                $(".delete_button_switcher").addClass("delete_mode");
                $(".delete_lesson").show();
            }
        });

        $.datepicker.setDefaults(
            $.extend(
                {'dateFormat':'dd-mm'},
                $.datepicker.regional['ru']
            )
        );

        this.setupLessonEditing();
    }

    setAttendance($cell) {
        let successText = 'Присутствие установлено';
        let errorText = 'Не удалось установить присутствие';
        let attendance = 1;
        if (!$cell.find('input').prop('checked')) {
            successText = 'Отсутствие установлено';
            errorText = 'Не удалось установить отсутствие';
            attendance = 0;
        }

        this.sendAttendance($cell, 'handler/journal/SetAttendance', {attendance},
            $input => $input.prop("checked", !$input.prop("checked")), successText, errorText);
    }

    sendAttendance($cell, url, data, reset, successText, errorText) {
        let $input = $cell.children('input');

        $input.turnOff();

        data.recordBookID = this.cell.recordBook;
        data.disciplineID = this.settings.discipline.id;
        data.lessonID = this.cell.lesson;

        $.postJSON(g_URLdir + url, data).success(() => {
            this.recountScores($cell);
            Popup.success(successText);
        }).fail(jqXHR => {
            reset($input);
            let status = parseInt(jqXHR.status);
            try {
                if (status != 400) throw null;
                let message = JSON.parse(jqXHR.responseText).message;
                if (!message) throw null;
                Popup.error(message);
            } catch (error) {
                Popup.error(errorText);
            }
        }).always(() => $input.turnOn());
    }

    recountAllScores() {
        let self = this;
        $('#studentsAttendance').children('tbody').children().each(function() {
            self.recountScoresRow($(this));
        });
    }

    recountScores($cell) {
        let $row = $cell.parent();
        this.recountScoresRow($row);
    }

    recountScoresRow($row) {
        let $cells = $row.children();

        $regularCells = $cells.filter('.regular');
        if ($regularCells.length) {
            let semesterRate = $regularCells.aggregate(function(init, elem) {
                if ($(elem).css('display') == 'none') {
                    return init;
                } else {
                    return init + Base.parseRate($(elem));
                }
            });
            $cells.filter('.semester').text(semesterRate);
        }
    }

    setupLessonEditing() {

        let self = this;
        const editLessonDate = function ($cell, lessonID, dateText, onSuccess, onFail) {
            $cell.turnOff();

            let currentYear = self.settings.discipline.semesterYear;
            if (self.settings.discipline.semesterNum === 2) {
                currentYear += 1;
            }
            let data = {};
            data.lessonID = lessonID;
            formattedDate = dateText.split('-').reverse().join('-');
            data.date = currentYear+"_"+formattedDate;

            $.postJSON(g_URLdir + "handler/journal/SetLessonDate", data).success(() => {
                Popup.success("Дата успешно установлена");
                onSuccess();
            }).fail(jqXHR => {
                onFail();
                let status = parseInt(jqXHR.status);
                try {
                    if (status != 400) throw null;
                    let message = JSON.parse(jqXHR.responseText).message;
                    if (!message) throw null;
                    Popup.error(message);
                } catch (error) {
                    Popup.error("Не удалось сменить дату занятия");
                }
            }).always(() => $cell.turnOn());
        };

        const createLesson = function (disciplineID, dateText, lessonType, onSuccess, onFail) {
            let data = {};
            data.disciplineID = disciplineID;
            data.date = dateText;
            data.lessonType = lessonType;

            $.postJSON(g_URLdir + "handler/journal/CreateLesson", data).success((response) => {
                Popup.success("Занятие успешно создано");
                onSuccess(response["lessonID"]);
            }).fail(jqXHR => {
                onFail();
                let status = parseInt(jqXHR.status);
                try {
                    if (status != 400) throw null;
                    let message = JSON.parse(jqXHR.responseText).message;
                    if (!message) throw null;
                    Popup.error(message);
                } catch (error) {
                    Popup.error("Не удалось создать занятие");
                }
            });
        };

        const deleteLesson = function (lessonID, onSuccess, onFail) {
            let data = {};
            data.lessonID = lessonID;

            $.postJSON(g_URLdir + "handler/journal/DeleteLesson", data).success((response) => {
                Popup.success("Занятие успешно удалено");
                onSuccess(response["lessonID"]);
            }).fail(jqXHR => {
                onFail();
                let status = parseInt(jqXHR.status);
                try {
                    if (status != 400) throw null;
                    let message = JSON.parse(jqXHR.responseText).message;
                    if (!message) throw null;
                    Popup.error(message);
                } catch (error) {
                    Popup.error("Не удалось удалить занятие");
                }
            });
        };

        const onDateInput = function($elem, dateText, oldDateText) {
            //console.log($elem);
            //console.log(dateText);
            //console.log(oldDateText);
            if (dateText !== oldDateText) {
                let lessonID = Base.parseid($elem);
                editLessonDate($elem, lessonID, dateText, function() {

                }, function() {
                    $elem.val(oldDateText);
                });
            }
        }

        let currentYear = self.settings.discipline.semesterYear;
        if (self.settings.discipline.semesterNum === 2) {
            currentYear += 1
        }

        const defaultDate = new Date(Date.now());
        let numericYear = parseInt(currentYear);
        if (self.settings.discipline.semesterNum === "2") {
            numericYear++;
        }
        defaultDate.setFullYear(numericYear);

        let $cell = undefined;
        const setDateTimePickerHandlers = function ($elem) {
            let oldDateText = $elem.val();
            $cell = $elem;
            //$elem.datetimepicker({
            $elem.datepicker({
                dateFormat: 'dd-mm',
                changeMonth: true,
                changeYear: false,
                defaultDate: defaultDate,
                //timeFormat: 'HH:mm:ss',
                beforeShow: function() {
                    oldDateText = $(this).val();
                    $cell = this;
                },
                onClose: function(dateText) {
                    onDateInput($(this), dateText, oldDateText);
                }
            });
            // $elem.on("change", {}, $elem.dattetimepicker()) ломает datetimepicker в Chrome:
            // jquery-1.11.1.min.js:3 Uncaught TypeError: ((m.event.special[e.origType] || {}).handle || e.handler).apply is not a function
            //     at HTMLInputElement.dispatch (jquery-1.11.1.min.js:3)
            //     at HTMLInputElement.r.handle (jquery-1.11.1.min.js:3)
            //     at Object.trigger (jquery-1.11.1.min.js:3)
            //     at HTMLInputElement.<anonymous> (jquery-1.11.1.min.js:3)
            //     at Function.each (jquery-1.11.1.min.js:2)
            //     at m.fn.init.each (jquery-1.11.1.min.js:2)
            //     at m.fn.init.trigger (jquery-1.11.1.min.js:3)
            //     at Timepicker._updateDateTime (jquery-timepicker-addon-1.6.3.js:987)
            //     at Datepicker.$.datepicker._formatDate (jquery-timepicker-addon-1.6.3.js:1808)
            //     at Datepicker._selectDay (jquery-ui.js:8191)
            // $elem.on("change", {}, $elem.datetimepicker());
            //$elem.change().datetimepicker();
            $elem.change().datepicker();
        };
        setDateTimePickerHandlers($(".lesson_input"));

        const setLessonNameHandlers = function ($elem) {
            let self = this;
            const editLessonName = function ($cell, lessonID, newName, onSuccess, onFail) {
                $cell.turnOff();

                let data = {};
                data.lessonID = lessonID;
                data.name = newName;

                $.postJSON(g_URLdir + "handler/journal/SetLessonName", data).success(() => {
                    Popup.success("Название успешно установлено");
                    onSuccess();
                }).fail(jqXHR => {
                    onFail();
                    let status = parseInt(jqXHR.status);
                    try {
                        if (status != 400) throw null;
                        let message = JSON.parse(jqXHR.responseText).message;
                        if (!message) throw null;
                        Popup.error(message);
                    } catch (error) {
                        Popup.error("Не удалось сменить название занятия");
                    }
                }).always(() => $cell.turnOn());
            };

            let setLessonName = function(lessonID, oldName, newName) {
                if (newName !== oldName) {
                    editLessonName($elem, lessonID, newName, function() {
                    }, function() {
                        $elem.val(oldName);
                    });
                }
            };

            let oldNameText = $elem.val();
            $elem.on("change", function() {
                let lessonID = Base.parseid($(this));
                let newNameText = $(this).val();
                setLessonName(lessonID, oldNameText, newNameText);
            });
        };
        setLessonNameHandlers($(".lesson_name_input"));

        // третий параметр - это костыль для перевода в формат даты, поддерживаемый по умолчанию БД и
        // хранимой процедурой create_lesson
        // см. http://gitlab.mmcs.sfedu.ru/it-lab/grade/issues/376
        const onNewLessonDate = function($cell, newLessonDate, formattedLessonDate) {
            //console.log(newLessonDate);
            const newDate = currentYear+"-"+formattedLessonDate;
            //console.log(Base.parseClass($cell.parent()));
            const buttonColumn =Base.parseClass($cell.parent());
            const newButtonColumn = buttonColumn + 1;
            const lastColumn = buttonColumn - 1;
            const $table = $("#studentsAttendance");
            const newColumn = buttonColumn;
            let newLessonType = parseInt($('#lessonSelector').val());
            if (newLessonType === 0) {
                newLessonType = 1;
            }

            //
            createLesson(self.settings.discipline.id, newDate, newLessonType, function(newLessonID) {
                $table.find('tr').each(function(){
                    $cell.parent().removeClass("col_" + buttonColumn);
                    $cell.parent().addClass("col_" + newButtonColumn);
                    var $trow = $(this);
                    let toClone = $trow.find('td').eq(lastColumn);
                    let clone = toClone.clone();
                    let row = $trow.index();// + 1;
                    $trow.children(".journal.empty").attr("id", "col_row_"+(newColumn+1)+"_"+row);
                    toClone.find("input").attr("id", "checkbox_col_row_"+newColumn+"_"+row);
                    toClone.find(".checkmark").attr("for", "checkbox_col_row_"+newColumn+"_"+row);
                    if($trow.index() === 0) {
                        toClone.removeClass("lesson_col_"+lastColumn);
                        toClone.addClass("lesson_col_"+newColumn);
                        clone.attr("id", "lesson_"+newLessonID);
                        toClone.children("input").removeClass("col_"+lastColumn);
                        toClone.children("input").addClass("col_"+newColumn);
                        clone.children(".lesson_input").attr("id","lesson_input_"+newLessonID);
                        clone.children(".lesson_input").val(newLessonDate);
                        clone.find(".lesson_name_input").attr("id","lesson_name_input_"+newLessonID);
                        setDateTimePickerHandlers(clone.children(".lesson_input"));
                        setLessonNameHandlers(clone.find(".lesson_name_input"));
                    }
                    else {
                        toClone.attr("id", "col_row_"+newColumn+"_"+row);
                        clone.attr("id","col_row_"+lastColumn+"_"+row);
                        clone.children("input").prop("checked", false);

                    }
                    clone.addClass("lesson_type_"+newLessonType);
                    clone.toggle();
                    toClone.before(clone);

                });
            }, function() {
            });
        };

        let decrementColumn = function($cell, from) {
            if (!$cell) return;
            let columnMatch = null;
            if ($cell.attr('class')) {
                columnMatch = $cell.attr('class').match(/(lesson_)?col_(\d+)/);
            }
            if (columnMatch) {
                const thisColumn = parseInt(columnMatch[2]);
                if (thisColumn <= from) {
                    return;
                }
                const newColumn = thisColumn - 1;
                const newColumnClass = (columnMatch[1] ? "lesson_":"") + "col_" + newColumn;
                $cell.removeClass(columnMatch[0]);
                $cell.addClass(newColumnClass);
            } else if ($cell.attr('id')){
                let columnMatch = $cell.attr('id').match(/(checkbox_)?col_row_(\d+)_(\d+)/);
                if (!columnMatch) {
                    return;
                }
                    const thisColumn = parseInt(columnMatch[2]);
                    if (thisColumn <= from) {
                        return;
                    }
                    const newColumn = thisColumn - 1;
                    const newColumnId = (columnMatch[1] ? "checkbox_":"") + "col_row_" + newColumn + "_"+columnMatch[3];
                    $cell.attr('id', newColumnId);
                } else {
                    if (!$cell.attr('for')) return;
                    let columnMatch = $cell.attr('for').match(/(checkbox_)?col_row_(\d+)_(\d+)/);
                    if (columnMatch) {
                        const thisColumn = parseInt(columnMatch[2]);
                        if (thisColumn <= from) {
                            return;
                        }
                        const newColumn = thisColumn - 1;
                        const newColumnFor = (columnMatch[1] ? "checkbox_" : "") + "col_row_" + newColumn + "_" + columnMatch[3];
                        $cell.attr('for', newColumnFor);
                    }
                }
        };

        const onLessonDelete = function($cell) {
            if (!confirm('Вы уверены, что хотите удалить это занятие?')) {
                return;
            }
            const lessonID = Base.parseid($cell.parent());
            const column = Base.parseClass($cell.parent());

            const $table = $("#studentsAttendance");
            $table.find('tr').each(function() {
               var $trow = $(this);
               let toDec = $trow.find('td');
               toDec.each(function() {
                   decrementColumn($(this), column);
                   decrementColumn($(this).find("input"), column);
                   decrementColumn($(this).find("label"), column);
               });
            });

            deleteLesson(lessonID, function(newLessonID) {
                $table.find('tr').each(function(){
                    $(this).children().filter("td:eq("+ column + ")").remove();
                    self.recountScoresRow($(this));
                });
            }, function() {
            });
        };

        $('#studentsAttendance').on('click', '.delete_lesson', function () {
            onLessonDelete($(this));
        });

        //$( "#hiddenField" ).datetimepicker({
        $( "#hiddenField" ).datepicker({
            showOn: "button",
            buttonText: "Новое занятие",
            dateFormat: 'dd-mm',
            changeMonth: true,
            changeYear: false,
            defaultDate: defaultDate,
            //timeFormat: 'HH:mm:ss',
            onClose: function(newLessonDate) {
                // это костыль для перевода в формат даты, поддерживаемый по умолчанию БД и
                // хранимой процедурой create_lesson
                // см. http://gitlab.mmcs.sfedu.ru/it-lab/grade/issues/376
                const formattedLessonDate = newLessonDate.split('-').reverse().join('-');
                onNewLessonDate($(this), newLessonDate, formattedLessonDate);
            }
        }).next(".ui-datepicker-trigger").addClass("defaultForm BlueButton newLessonButton");
    }
}

class GroupFilter {
    constructor(settings, $table, $selector) {
        this.settings = settings;
        this.$table = $table.children('tbody');

        let groupFilter = JSON.parse(localStorage.groupFilter || "{}");
        let groupid = groupFilter[this.settings.discipline.id] || 0;

        $selector.children('[value=' + groupid + ']').prop('selected', true);
        this.filterGroups(groupid);

        this.listen($selector);
    }

    filterGroups(groupid) {
        let $trs = this.$table.children();
        $trs.show();

        if (groupid) {
            $trs.not(`.group_${groupid}`).hide();
        }
    }

    listen($selector) {
        $selector.change(() => {
            let groupid = parseInt($selector.val());

            let groupFilter = JSON.parse(localStorage.groupFilter || "{}");
            groupFilter[this.settings.discipline.id] = groupid;
            localStorage.groupFilter = JSON.stringify(groupFilter);

            this.filterGroups(groupid);
        });
    }
}

class LessonFilter {
    constructor(settings, $table, $selector, journal) {
        this.settings = settings;
        this.$table = $table;
        this.parentJournal = journal;

        let lessonFilter = JSON.parse(localStorage.lessonFilter || "{}");
        let lesson_type_id = lessonFilter[this.settings.discipline.id] || 0;

        $selector.children('[value=' + lesson_type_id + ']').prop('selected', true);
        this.filterLessons(lesson_type_id);

        this.listen($selector);
    }

    filterLessons(lesson_type_id) {

        let $trs = this.$table.find('.journal.checkbox.regular');
        $trs.show();
        $('#journalHead .lesson_input').parent().show();
        if (lesson_type_id) {
            $('#journalHead .lesson_input').parent().not(`.lesson_type_${lesson_type_id}`).hide();
            $trs.not(`.lesson_type_${lesson_type_id}`).hide();
        }
        $("#lesson_0").hide();
        let hidden_col = Base.parseClass($("#lesson_0"));
        $('[id^="col_row_'+hidden_col+'"]').hide();
        this.parentJournal.recountAllScores();
    }

    listen($selector) {
        $selector.change(() => {
            let lesson_type_id = parseInt($selector.val());

            let lessonTypeFilter = JSON.parse(localStorage.lessonFilter || "{}");
            lessonTypeFilter[this.settings.discipline.id] = lesson_type_id;
            localStorage.lessonFilter = JSON.stringify(lessonTypeFilter);

            this.filterLessons(lesson_type_id);
        });
    }
}

$(() => new Journal());
